function L = assemLoad_BFE(Mesh,QuadRule,FHandle,varargin)
% ASSEMLOAD_BFE Assemble bilinear FE contributions.
%
%   L = ASSEMLOAD_BFE(MESH,QUADRULE,FHANDLE) assembles the global load 
%   vector for the load data given by the function handle EHANDLE.
%
%   The struct MESH must at least contain the following fields:
%    COORDINATES  M-by-2 matrix specifying the vertices of the mesh.
%    ELEMENTS     N-by-4 matrix specifying the elements of the mesh.
%    ELEMFLAG     N-by-1 matrix specifying additional element information.
%
%   QUADRULE is a struct, which specifies the Gauss qaudrature that is used
%   to do the integration:
%    W Weights of the Gauss quadrature.
%    X Abscissae of the Gauss quadrature.
%
%   L = ASSEMLOAD_BFE(COORDINATES,QUADRULE,FHANDLE,FPARAM) also handles the
%   additional variable length argument list FPARAM to the function handle
%   FHANDLE.
%
%   Example:
%
%   FHandle = @(x,varargin)x(:,1).^2+x(:,2).^2;
%   L = assemLoad_BFE(Mesh,TProd(gauleg(0,1,2)),FHandle);
%
%   See also shap_BFE.

%   Copyright 2005-2005 Patrick Meury & Mengyu Wang
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum
%   CH-8092 Zurich, Switzerland

  % Initilaize constants
  
  nCoordinates = size(Mesh.Coordinates,1);
  nElements = size(Mesh.Elements,1);
  
  % Preallocate memory
  
  L = zeros(nCoordinates,1);
  
  % Assemble element contributions
  
  for i = 1:nElements
    
    % Extract vertex numbers
    
    vidx = Mesh.Elements(i,:);

    % Compute local load vector

    floc = Load_BFE(Mesh.Coordinates(vidx,:), FHandle, QuadRule);

    % Add into global load vector
  
    L(vidx) = L(vidx) + floc;
      
  end
  
return
